globals 
[
  clock vsplit vclock             ;; clock variables
  box-x box-y                     ;; patch coords of box's upper right corner
]

turtles-own 
[
  speed  kinetic-energy  mass           ;; turtle info
  v1t v1l tmp-turtle                    ;; collision info (turtle 1)
  heading2 mass2 speed2 v2t v2l turtle2 ;; collision info (turtle 2)
  theta                                 ;; collision info (both turtles)
]

to setup
  ca
  set-default-shape turtles "circle"
  set clock 0
  set vclock 0
end

to update-variables
  ask turtles
    [ set kinetic-energy (0.5 * speed * speed * mass) ]
  set vsplit ceiling max values-from turtles [speed]
end

to recalculate-vsplit
  ;; this needs to be done without-interruption to be sure that nothing tries to
  ;; use vsplit or vclock before they've been recalculated
  without-interruption
  [
    set vsplit 2 * vsplit
    set vclock 2 * vclock
  ]
end

to go
  if bounce?
    [ ask turtles [ bounce ]]
  without-interruption 
    [ ask turtles [ move ] ]
  set vclock (vclock + 1)
  if (vclock = vsplit)
  [
    set clock (clock + 1)
    set vclock 0
    update-variables
  ]
  ;; stop if all particles have left the screen
  if not any? turtles
    [ stop ]
end

to bounce  ;; turtle procedure
  locals [new-px new-py]
  ;; if we're not about to hit a wall (yellow patch), or if we're already on a
  ;; wall, we don't need to do any further checks
  if pcolor = yellow or pcolor-of patch-at dx dy != yellow 
    [ stop ]
  ;; get the coordinates of the patch we'll be on if we go forward 1
  set new-px round (xcor + dx)
  set new-py round (ycor + dy)
  ;; if hitting left or right wall, reflect heading around x axis
  if (abs new-px = box-x)
    [ set heading (- heading) ]
  ;; if hitting top or bottom wall, reflect heading around y axis
  if (abs new-py = box-y)
    [ set heading (180 - heading) ]
end

to move  ;; turtle procedure
  locals [ next-patch ]
  while [(speed / vsplit) >= 1.0]
    [ recalculate-vsplit ]
  set next-patch patch-at ((speed / vsplit) * dx) ((speed / vsplit) * dy)
  ;; die if we're about to wrap...
  if ((pxcor != 0 and (pxcor-of next-patch) = (- pxcor)) or
      (pycor != 0 and (pycor-of next-patch) = (- pycor)))
    [ die ]
  jump (speed / vsplit)
  if collisions? 
    [ check-for-collision ]
end

to check-for-collision  ;; turtle procedure
  if count other-turtles-here >= 1
  [
    set tmp-turtle random-one-of other-turtles-here
    if ((who > who-of tmp-turtle) and (turtle2 != tmp-turtle))
      [ collide ]
  ]
end
        
to collide  ;; turtle procedure
  get-turtle2-info
  calculate-velocity-components
  set-new-speed-and-headings
end

to get-turtle2-info  ;; turtle procedure
  set turtle2 tmp-turtle
  set mass2 mass-of turtle2
  set speed2 speed-of turtle2
  set heading2 heading-of turtle2
end

to calculate-velocity-components
  locals [vcm]  ;; CM vel. along dir. theta
  set theta (random-int-or-float 360)
  set v1l (speed * sin (theta - heading))
  set v1t (speed * cos (theta - heading))
  set v2l (speed2 * sin (theta - heading2))
  set v2t (speed2 * cos (theta - heading2))
  set vcm (((mass * v1t) + (mass2 * v2t)) / (mass + mass2))
  set v1t (vcm + vcm - v1t)
  set v2t (vcm + vcm - v2t)
end

to set-new-speed-and-headings  ;; turtle procedure
  locals [ temp-speed ]
  set temp-speed speed
  set speed sqrt ((v1t * v1t) + (v1l * v1l))
  set heading (theta - (atan v1l v1t))
  set speed-of turtle2 sqrt ((v2t * v2t) + (v2l * v2l))
  set heading-of turtle2 (theta - (atan v2l v2t))
  recolor
  ask turtle2 
    [ recolor ]
end

to recolor  ;; turtle procedure
  ifelse ( speed < (0.5 * 10) and speed > 0 )
  [
    set color blue
  ]
  [
    ifelse speed > (1.5 * 10)
      [ set color red ]
      [ set color green ]
  ]
end

;;;
;;; drawing procedures
;;;

;; allows the user to place the box by clicking on one corner.
to place-box
  undraw-box
  set pcolor-of patch 0 0 grey
  while [not mouse-down?] [wait 0.01]
  set box-x (abs round mouse-xcor)
  set box-y (abs round mouse-ycor)
  ifelse box-x > 1 and box-y > 1
    [ draw-box ]
    [ print "The box must be larger." ]
  while [mouse-down?] [wait 0.1]
  set pcolor-of patch 0 0 black
end

;; removes the box.
to undraw-box
  ask patches [ set pcolor black ]
end

;; draws a square yellow box at distance box-edge from the origin
to draw-box
  ask patches with [((abs pxcor = box-x) and (abs pycor <= box-y)) or
                    ((abs pycor = box-y) and (abs pxcor <= box-x))]
    [ set pcolor yellow ]
end

;; allows the user to place a particle using the mouse
to place-particles
  if mouse-down?
  [
    create-particles number mouse-xcor mouse-ycor
    while [mouse-down?] [wait 0.1]
  ]
end

;; places n particles in a cluster around point (x,y).
to create-particles [n x y]
  cct n
  [
    setxy x y
    set speed 10
    set mass 2
    set heading random-int-or-float 360.0
    ;; if we're only placing one particle, use the exact position
    if n > 1
      [ jump random-int-or-float 5.0 ]
    set kinetic-energy (0.5 * speed * speed * mass)
    recolor
  ]
  set vsplit ceiling max values-from turtles [speed]
end

; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo GasLab Gas in a Box model.
; http://ccl.northwestern.edu/netlogo/models/GasLabGasinaBox.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/GasLabGasinaBox
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
280
12
645
398
35
35
5.0
1
10
1
1
1

CC-WINDOW
14
285
237
410
Command Center

MONITOR
18
230
99
279
clock
clock
0
1

SLIDER
14
47
227
80
number
number
1
100
10
1
1
NIL

BUTTON
14
124
89
157
go
go
T
1
T
OBSERVER
T

BUTTON
14
10
89
43
NIL
setup
NIL
1
T
OBSERVER
T

TEXTBOX
13
166
136
185
Interactions

SWITCH
17
187
127
220
collisions?
collisions?
1
1
-1000

SWITCH
130
187
234
220
bounce?
bounce?
1
1
-1000

MONITOR
108
231
237
280
number of particles
count turtles
0
1

BUTTON
14
84
146
117
NIL
place-particles
T
1
T
OBSERVER
T

BUTTON
106
10
207
43
NIL
place-box
NIL
1
T
OBSERVER
T

@#$#@#$#@
Connected Chemistry Curriculum is currently under development, for more information please refer to http://ccl.northwestern.edu/curriculum/chemistry/


WHAT IS IT?
-----------
This model simulates the behavior of gas particles in a changeable container.  The Kinetic Molecular Theory (KMT) underlies the rules that the particles follow, and some of these rules can be added or taken away from the model.  Whether particles are contained in a box, bounce off its walls or collide with each other can be determined.

This model is part of the Connected Chemistry High School Curriculum and connects students prior learning about pressure with the Kinetic Molecular Theory.  This is one of a number of models that have been adapted from the model Gas in a Box, which can be found in the GasLab folder under Chemistry and Physics in the models library.  

In all of the Connected Chemistry Curriculum models, the same basic rules are used for expressing what happens when gas particles collide.  Each model has different features in order to show different aspects of the behavior of gases.


HOW IT WORKS
------------
Particles have kinetic energy (that which is due to their motion).  If the particles collide among themselves, these collisions are perfectly elastic.  Particles are colored according to their speed -- blue for slow, green for medium, and red for high speeds.

When choosing to have the particles collide, they do so, conserving energy and momentum, in the following way:
1. Two particles "collide" if they find themselves on the same patch.
2. A random axis is chosen, as if they are two balls that hit each other and this axis is the line connecting their centers.
3. They exchange momentum and energy along that axis, according to the conservation of momentum and energy.  This calculation is done in the center of mass system.
4. Each turtle is assigned its new velocity, energy, and heading.

Choosing that the model will have a container, and deciding to have the particles bounce off the walls means that these collisions are elastic, or that if a turtle finds itself on or very close to a wall of the container, it "bounces" -- that is, reflects its direction and keeps its same speed.


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2002).  NetLogo Chem KMT model.  http://ccl.northwestern.edu/netlogo/models/ChemKMT. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2002 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/ChemKMT for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

circle
false
0
Circle -7566196 true true 35 35 230

@#$#@#$#@
NetLogo 2.0beta5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
