breeds [
  water          ;; water molecules
  hydronium      ;; hydronium ions (green)
  hydroxide      ;; base molecules (red)
  acid           ;; parent acid molecules (yellow)
  conj-base      ;; conjugate base molecules of parent acid (orange)
]

globals [
  pH
]

to setup
  ca
  set-default-shape water "molecule2"
  set-default-shape hydronium "molecule3"
  set-default-shape hydroxide "molecule1"
  set-default-shape acid "molecule2"
  set-default-shape conj-base "molecule1"
  create-custom-acid starting-acid  ;; creates variable amount of acid
    [ set color yellow ]
  ask acid [
    ;; initializes amount of dissociated acid
    if (random-float 100) > 98
      [ dissociate ] ]
  if see-starting-water?
    [ create-custom-water 100    ;; creates constant volume of water
        [ set color blue ] ]
  create-custom-conj-base starting-conj-base  ;; creates variable amount of conjugate base
    [ set color orange ]
  ask turtles                   ;; randomize position and heading of turtles
    [ setxy (random-float screen-size-x)
            (random-float screen-size-y)
      rt random-float 360 ]
  calculate-pH
  plot-pH
end

;; dissociate an acid molecule into hydronium and conj-base
;; this should only be called by turtles that are acids
to dissociate  ;; turtle procedure
  hatch 1 [
    set breed hydronium
    set color green ]
  set breed conj-base
  set color orange
end

to go
  ;; without-interruption is used to prevent molecules from
  ;; getting involved in more than one reaction simultaneously
  ask hydroxide [ 
    without-interruption [
      react-hydroxide ] ]
  ask hydronium [
    without-interruption [
      react-hydronium ] ]
  ask turtles
    [ fd 1
      rt (random-float 10 - random-float 10) ]    ;; moves turtles randomly around the screen
  calculate-pH
  plot-pH
  plot-molecules
end

to react-hydroxide  ;; hydroxide procedure
  locals [ partner ]
  set partner random-one-of hydronium-here
  ifelse partner != nobody
    [ react partner ]
    [ set partner random-one-of acid-here
      if partner != nobody
        [ react partner ] ]
end

to react-hydronium  ;; hydronium procedure
  locals [ partner ]
  set partner random-one-of hydroxide-here
  ifelse partner != nobody
    [ react partner ]
    [ set partner random-one-of conj-base-here
      if partner != nobody
        [ react partner ] ]
end

to react [ partner ]  ;; turtle procedure
  ;; all hydroxide molecules will instantly react with
  ;; hydronium molecules and form water
  ifelse ((breed-of partner) = hydronium) or ((breed-of partner) = hydroxide)
    [ set breed-of partner water
      set color-of partner blue
      set breed water
      set color blue ]
    [ ;; hydroxide molecules have a high probability of
      ;; reacting with acid molecules
      ifelse ((breed-of partner) = acid)
        [ if ((random-float 100) > 80)
          [ set breed-of partner conj-base
            set color-of partner orange
            set breed water
            set color blue ] ]
        ;; hydronium ions will react with conjugate base to form acid
        [ if ((breed-of partner) = conj-base)
          [ if ((random-float 100) > 80)
            [ set breed-of partner acid
              set color-of partner yellow
              set breed water
              set color blue ] ] ] ]
end

;; calculates the pH from the amount of the various ions in solution;
;; note that for simplicity the calculations don't take the true molar
;; concentration of water into account; instead we simply divide by
;; a factor of 1000 to bring the numbers into a reasonable range
to calculate-pH
  locals [ volume concH concOH pOH ]
  set volume count turtles 
  set concH (count hydronium / volume)
  set concOH (count hydroxide / volume)
  ifelse (concH = concOH)
    [ set pH 7 ]
    [ ifelse (concH > concOH)
      [ set pH (- log (concH / 1000) 10) ]
      [ set pOH (- log (concOH / 1000) 10)
        set pH 14 - pOH ] ]
end

;; add more base to the system
to add-base
  create-custom-hydroxide added-base [
    set color red
    fd 1
    rt random-float 360 ]
end

;; add more acid to the system
to add-acid
  create-custom-hydronium added-acid [
    set color green
    fd 1
    rt random-float 360 ]
end

;; plotting procedures

to plot-pH
  set-current-plot "pH Curve"
  plot pH
end

to plot-molecules
  set-current-plot "Molecule Counts"
  set-current-plot-pen "conj. base"
  plot count conj-base
  set-current-plot-pen "acid"
  plot count acid
  set-current-plot-pen "hydroxide"
  plot count hydroxide
  set-current-plot-pen "hydronium"
  plot count hydronium
  set-current-plot-pen "water"
  plot count water
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
;
; Copyright 2001 by Uri Wilensky.  Updated 2004.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2001).  NetLogo Buffer model.
; http://ccl.northwestern.edu/netlogo/models/Buffer.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/Buffer
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
279
10
664
416
12
12
15.0
1
10
1
1
1

CC-WINDOW
665
85
926
220
Command Center

SLIDER
17
128
260
161
starting-conj-base
starting-conj-base
1
200
100
1
1
molecules

SLIDER
84
170
277
203
added-acid
added-acid
1
100
15
1
1
molecules

BUTTON
20
49
121
82
setup
setup
NIL
1
T
OBSERVER
T

BUTTON
132
49
262
82
go
go
T
1
T
OBSERVER
T

BUTTON
7
170
79
203
add acid
add-acid
NIL
1
T
OBSERVER
T

BUTTON
7
213
79
246
add base
add-base
NIL
1
T
OBSERVER
T

PLOT
7
299
264
495
pH Curve
Time
pH
0.0
100.0
0.0
14.0
true
false
PENS
"default" 1.0 0 -65536 true

SLIDER
84
213
277
246
added-base
added-base
1
100
15
1
1
molecules

PLOT
279
427
873
646
Molecule Counts
time
#
0.0
100.0
0.0
10.0
true
true
PENS
"hydronium" 1.0 0 -11352576 true
"conj. base" 1.0 0 -44544 true
"hydroxide" 1.0 0 -65536 true
"acid" 1.0 0 -2500352 true
"water" 1.0 0 -16776961 true

SLIDER
17
92
261
125
starting-acid
starting-acid
1
200
100
1
1
molecules

SWITCH
42
258
216
291
see-starting-water?
see-starting-water?
1
1
-1000

MONITOR
37
568
119
617
pH
pH
2
1

MONITOR
37
515
119
564
# hydronium
count hydronium
0
1

MONITOR
133
514
212
563
# hydroxide
count hydroxide
0
1

@#$#@#$#@
WHAT IS IT?
-----------
This model demonstrates the behavior of a buffered solution.  A buffer is a solution that resists change in pH when either acid or base are added into it, within limits.  It is best viewed as the third model in the ACID-BASE package. 

Chemists and biologists use the properties of acids and bases to create buffer solutions.  It is often desirable to keep the acidity, or hydronium ion concentration of a solution, as nearly constant as possible.  Examples of such situation would be: measuring the velocity of a reaction, selectively dissolving certain salts, or studying the growth of bacteria or plants.   In our body, the blood has a buffering capacity, keeping the pH at 7.35.  If the pH goes above 7.7 (alkalosis) or below 7.0 (acidosis), the results are fatal.

To accomplish this feat, buffers depend on the presence of both a weak acid and its conjugate base. With both of these species in the solution, additional acids and bases are neutralized according to the following chemical equations. H-A denotes the weak acid and A- denotes its conjugate base.

|           -                      -           -       +
| H-A  +  OH     ----->   H O  +  A           A  +  H O    ------->   H O  +  H-A
|                          2                         3                 2

The pH of a buffer is determined in the same manner that the pH of a weak acid is determined - by counting the number of hydronium and hydroxide ions and calculating their ratio with total number of ions and molecules in the solution.

When there are more hydroniums,  pH = - log ( hydronium concentration )

When there are more hydroxides,  pH = 14 - pOH = 14 - log ( hydroxide concentration )

Buffers are effective only within their unique buffering range. This range is determined by the concentrations of the weak acid and its conjugate base.  Outside of its buffering range, the solution behaves as a strong acid or base.


HOW TO USE IT
--------------
Decide how many acid molecules should be present at the start of the simulation with the STARTING-ACID slider. Set the number of conjugate base molecules with the STARTING-CONJ-BASE slider.    

Press SETUP.  The turtles will distribute randomly across the screen. 

YELLOW turtles are acid molecules (HA)
ORANGE turtles are conjugate base molecules (A-)
GREEN turtles represent hydronium ions (H30+) 
RED turtles are hydroxide molecules (OH-)
BLUE turtles represent water molecules (H20)

In this model we are assuming that 2% of the original acid molecules are dissociated into 2 conjugate base molecules and 2 hydronium molecules.  This is true only before the molecules start interacting between themselves.

Press GO. The molecules will move randomly across the screen.  

When two turtles occupy the same patch, the following rules apply:
1.  When a weak acid and a water molecule collide, the acid molecule dissociates into its conjugate base and the water molecule transforms to a hydronium ion.
2.  When hydroxide and hydronium ions collide, they always form two water molecules.
3.  When a weak acid and hydroxide collide, they have a high probability of transforming into a conjugate base and a water molecule.
4.  When hydronium and conjugate base collide, they have a high probability of transforming into a weak acid molecule and a water molecule.

To observe the effect of adding base or acid to the solution, set the number of acid molecules you want to add with the ADDED-ACID slider and press ADD-ACID (H30+)
Do the same for adding base with the ADDED-BASE slider and ADD-BASE button.

A number of plots and monitors can be observed:
The pH of the solution is plotted over time on the PH plot, and at each time tick on the PH monitor.  pH is calculated using the ratio of the number of hydronium and hydroxide molecules to the total number of turtles.  This is different from the chemistry calculation that relates this number to solution volume.
You can see the number of hydroniums and hydroxides in the solution in their monitors  (# HYDRONIUMS, # HYDROXIDES). 
You may follow the number of molecules of each species over time in the MOLECULE COUNTS plot.

You may choose to see the initial water molecules or not with the SEE-STARTING-WATER? switch.


THINGS TO NOTICE
----------------
After you press SETUP, the buffer solution is not yet in equilibrium.  After the model starts running, the molecules react until the system is equilibrated, and the pH doesn't change anymore.

Observe the pH curve.  How are the numbers of hydronium and hydroxide molecules in the monitors below related to the pH?

Examine the shape of the pH curve. Notice how the pH changes with respect to the amount of base or acid added. Does the buffer resist change in pH?

Observe the number of molecules in the plot.   When acid or base is added to the solution, they quickly disappear as they react. Can you determine which molecules react with each other?  What is the relationship between the two plots: pH and number of molecules?

What happens when large amounts of base or acid are added to the system?  Is it the same as adding small amounts?  Does the pH curve reflect this?


THINGS TO TRY
-------------
Add a large amount of acid or base to the solution and observe the effect.  Why does the pH change dramatically outside of the buffering range?  Would it be useful to add large amounts of base all at once to a solution in the laboratory if you were trying to adjust the pH?

Can you relate the idea of a "buffering range" to the molecules and their behavior in the model?

Try running the model with different amounts of acid or conjugate base.  The buffering range of the solution should shift.  How is the shift related to the changes you made?  Does changing the proportion between the number of weak acid and conjugate base molecules have an effect on the buffering capacity?

How does changing the amount of buffer molecules influence the buffering capacity?

Try pressing SETUP a number of times with the same initial settings, and observe the number of hydroniums.  Is the number always the same?  Press GO and watch the change in pH.  Why is the same equilibrium reached in each case?


EXTENDING THE MODEL
------------------
Increase the amount of hydronium originally generated upon SETUP by increasing the chance of dissociation in the procedures.  What effect does this have on the initial pH?

Add a button and procedure to add more conjugate base to the reaction.  How could this help a chemist who is trying to keep the pH of a solution at a constant value?

Add a procedure that allows you to plot a titration curve for a buffer.  Is it similar to any other titration curves you have seen?  What additional information can you learn from the titration curve?

Additional interactions could take place in a solution in addition to those stated in the rules.  These were omitted because their probability of occurring is low.  Try including one of these rules to the procedure.  For example, two water molecules could transform into a hydronium and hydroxide ions.  Does it make a change?


CREDITS AND REFERENCES
----------------------
Thanks to Mike Stieff for his work on this model.

To refer to this model in academic publications, please use: Wilensky, U. (2001).  NetLogo Buffer model. http://ccl.northwestern.edu/netlogo/models/Buffer. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2001 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/Buffer for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

molecule1
true
0
Circle -7566196 true true 80 80 142
Circle -1 true false 52 195 61

molecule2
true
0
Circle -7566196 true true 78 79 146
Circle -1 true false 40 41 69
Circle -1 true false 194 42 71

molecule3
true
0
Circle -7566196 true true 92 92 118
Circle -1 true false 120 32 60
Circle -1 true false 58 183 60
Circle -1 true false 185 183 60

@#$#@#$#@
NetLogo 2.0.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
